@@ -1,5 +1,5 @@
 import logging
-from typing import Any, Callable, Dict, Optional, Tuple
+from typing import Any, Container, Dict, Optional, Tuple
 from unittest import mock
 
 import numpy as np
@@ -11,16 +11,15 @@ from . import chunking, datamodel, geocoding, orbit, radiometry, scene
 logger = logging.getLogger(__name__)
 
 
+SPEED_OF_LIGHT = 299_792_458.0  # m / s
+
+
 def simulate_acquisition(
     dem_ecef: xr.DataArray,
     position_ecef: xr.DataArray,
-    slant_range_time_to_ground_range: Callable[
-        [xr.DataArray, xr.DataArray], xr.DataArray
-    ],
-    correct_radiometry: Optional[str] = None,
+    include_variables: Container[str] = (),
 ) -> xr.Dataset:
     """Compute the image coordinates of the DEM given the satellite orbit."""
-
     orbit_interpolator = orbit.OrbitPolyfitIterpolator.from_position(position_ecef)
     position_ecef = orbit_interpolator.position()
     velocity_ecef = orbit_interpolator.velocity()
@@ -28,27 +27,136 @@ def simulate_acquisition(
     acquisition = geocoding.backward_geocode(dem_ecef, position_ecef, velocity_ecef)
 
     slant_range = (acquisition.dem_distance**2).sum(dim="axis") ** 0.5
-    slant_range_time = 2.0 / geocoding.SPEED_OF_LIGHT * slant_range
+    slant_range_time = 2.0 / SPEED_OF_LIGHT * slant_range
 
     acquisition["slant_range_time"] = slant_range_time
 
-    maybe_ground_range = slant_range_time_to_ground_range(
-        acquisition.azimuth_time,
-        slant_range_time,
-    )
-    if maybe_ground_range is not None:
-        acquisition["ground_range"] = maybe_ground_range.drop_vars("azimuth_time")
-    if correct_radiometry is not None:
+    if include_variables and "gamma_area" in include_variables:
         gamma_area = radiometry.compute_gamma_area(
             dem_ecef, acquisition.dem_distance / slant_range
         )
         acquisition["gamma_area"] = gamma_area
 
-    acquisition = acquisition.drop_vars(["dem_distance", "satellite_direction", "axis"])
+    for data_var_name in acquisition.data_vars:
+        if include_variables and data_var_name not in include_variables:
+            acquisition = acquisition.drop_vars(data_var_name)  # type: ignore
+
+    # drop coordinates that are not associated with any data variable
+    for coord_name in acquisition.coords:
+        if all(coord_name not in dv.coords for dv in acquisition.data_vars.values()):
+            acquisition = acquisition.drop_vars(coord_name)  # type: ignore
 
     return acquisition
 
 
+def map_simulate_acquisition(
+    dem_ecef: xr.DataArray,
+    position_ecef: xr.DataArray,
+    template_raster: xr.DataArray,
+    correct_radiometry: Optional[str] = None,
+) -> xr.Dataset:
+    acquisition_template = xr.Dataset(
+        data_vars={
+            "slant_range_time": template_raster,
+            "azimuth_time": template_raster.astype("datetime64[ns]"),
+        }
+    )
+    include_variables = {"slant_range_time", "azimuth_time"}
+    if correct_radiometry is not None:
+        acquisition_template["gamma_area"] = template_raster
+        include_variables.add("gamma_area")
+
+    acquisition = xr.map_blocks(
+        simulate_acquisition,
+        dem_ecef,
+        kwargs={
+            "position_ecef": position_ecef,
+            "include_variables": include_variables,
+        },
+        template=acquisition_template,
+    )
+    return acquisition
+
+
+def do_terrain_correction(
+    product: datamodel.SarProduct,
+    dem_raster: xr.DataArray,
+    correct_radiometry: Optional[str] = None,
+    interp_method: xr.core.types.InterpOptions = "nearest",
+    grouping_area_factor: Tuple[float, float] = (3.0, 3.0),
+    radiometry_chunks: int = 2048,
+    radiometry_bound: int = 128,
+) -> tuple[xr.DataArray, Optional[xr.DataArray]]:
+    logger.info("pre-process DEM")
+
+    dem_ecef = xr.map_blocks(
+        scene.convert_to_dem_ecef, dem_raster, kwargs={"source_crs": dem_raster.rio.crs}
+    )
+    dem_ecef = dem_ecef.drop_vars(dem_ecef.rio.grid_mapping)
+
+    logger.info("simulate acquisition")
+
+    template_raster = dem_raster.drop_vars(dem_raster.rio.grid_mapping) * 0.0
+
+    acquisition = map_simulate_acquisition(
+        dem_ecef, product.state_vectors(), template_raster, correct_radiometry
+    )
+
+    simulated_beta_nought = None
+    if correct_radiometry is not None:
+        logger.info("simulate radiometry")
+
+        grid_parameters = product.grid_parameters(grouping_area_factor)
+
+        if correct_radiometry == "gamma_bilinear":
+            gamma_weights = radiometry.gamma_weights_bilinear
+        elif correct_radiometry == "gamma_nearest":
+            gamma_weights = radiometry.gamma_weights_nearest
+
+        acquisition = acquisition.persist()
+
+        simulated_beta_nought = chunking.map_ovelap(
+            obj=acquisition,
+            function=gamma_weights,
+            chunks=radiometry_chunks,
+            bound=radiometry_bound,
+            kwargs=grid_parameters,
+            template=template_raster,
+        )
+        simulated_beta_nought.attrs["long_name"] = "terrain-simulated beta nought"
+
+        simulated_beta_nought.x.attrs.update(dem_raster.x.attrs)
+        simulated_beta_nought.y.attrs.update(dem_raster.y.attrs)
+        simulated_beta_nought.rio.write_crs(dem_raster.rio.crs, inplace=True)
+
+    logger.info("calibrate image")
+
+    beta_nought = product.beta_nought()
+
+    logger.info("terrain-correct image")
+
+    # HACK: we monkey-patch away an optimisation in xr.DataArray.interp that actually makes
+    #   the interpolation much slower when indeces are dask arrays.
+    with mock.patch("xarray.core.missing._localize", lambda o, i: (o, i)):
+        geocoded = product.interp_sar(
+            beta_nought,
+            azimuth_time=acquisition.azimuth_time,
+            slant_range_time=acquisition.slant_range_time,
+            method=interp_method,
+        )
+
+    if correct_radiometry is not None:
+        assert simulated_beta_nought is not None
+        geocoded = geocoded / simulated_beta_nought
+        geocoded.attrs["long_name"] = "terrain-corrected gamma nought"
+
+    geocoded.x.attrs.update(dem_raster.x.attrs)
+    geocoded.y.attrs.update(dem_raster.y.attrs)
+    geocoded.rio.write_crs(dem_raster.rio.crs, inplace=True)
+
+    return geocoded, simulated_beta_nought
+
+
 def terrain_correction(
     product: datamodel.SarProduct,
     dem_urlpath: str,
@@ -68,9 +176,6 @@ def terrain_correction(
 
     :param product: SarProduct instance representing the input data
     :param dem_urlpath: dem path or url
-    :param orbit_group: overrides the orbit group name
-    :param calibration_group: overrides the calibration group name
-    :param coordinate_conversion_group: overrides the coordinate_conversion group name
     :param output_urlpath: output path or url
     :param correct_radiometry: default `None`. If `correct_radiometry=None`the radiometric terrain
     correction is not applied. `correct_radiometry=gamma_bilinear` applies the gamma flattening classic
@@ -79,12 +184,12 @@ def terrain_correction(
     'gamma_nearest' significantly reduces the processing time
     :param interp_method: interpolation method for product resampling.
     The interpolation methods are the methods supported by ``xarray.DataArray.interp``
-    :param multilook: multilook factor. If `None` the multilook is not applied
     :param grouping_area_factor: is a tuple of floats greater than 1. The default is `(1, 1)`.
     The `grouping_area_factor`  can be increased (i) to speed up the processing or
     (ii) when the input DEM resolution is low.
     The Gamma Flattening usually works properly if the pixel size of the input DEM is much smaller
-    than the pixel size of the input Sentinel-1 product. Otherwise, the output may have radiometric distortions.
+    than the pixel size of the input Sentinel-1 product.
+    Otherwise, the output may have radiometric distortions.
     This problem can be avoided by increasing the `grouping_area_factor`.
     Be aware that `grouping_area_factor` too high may degrade the final result
     :param open_dem_raster_kwargs: additional keyword arguments passed on to ``xarray.open_dataset``
@@ -98,6 +203,8 @@ def terrain_correction(
         raise ValueError(
             f"{correct_radiometry=}. Must be one of: {allowed_correct_radiometry}"
         )
+    if simulated_urlpath is not None and correct_radiometry is None:
+        raise ValueError("Simulation cannot be saved")
     if output_urlpath is None and simulated_urlpath is None:
         raise ValueError("No output selected")
 
@@ -124,63 +231,18 @@ def terrain_correction(
             f"{product.product_type=}. Must be one of: {allowed_product_types}"
         )
 
-    logger.info("pre-process DEM")
-
-    dem_ecef = xr.map_blocks(
-        scene.convert_to_dem_ecef, dem_raster, kwargs={"source_crs": dem_raster.rio.crs}
+    geocoded, simulated_beta_nought = do_terrain_correction(
+        product=product,
+        dem_raster=dem_raster,
+        correct_radiometry=correct_radiometry,
+        interp_method=interp_method,
+        grouping_area_factor=grouping_area_factor,
+        radiometry_chunks=radiometry_chunks,
+        radiometry_bound=radiometry_bound,
     )
-    dem_ecef = dem_ecef.drop_vars(dem_ecef.rio.grid_mapping)
-
-    logger.info("simulate acquisition")
-
-    template_raster = dem_raster.drop_vars(dem_raster.rio.grid_mapping) * 0.0
-    acquisition_template = xr.Dataset(
-        data_vars={
-            "slant_range_time": template_raster,
-            "azimuth_time": template_raster.astype("datetime64[ns]"),
-        }
-    )
-    if product.product_type == "GRD":
-        acquisition_template["ground_range"] = template_raster
-    if correct_radiometry is not None:
-        acquisition_template["gamma_area"] = template_raster
-
-    acquisition = xr.map_blocks(
-        simulate_acquisition,
-        dem_ecef,
-        kwargs={
-            "position_ecef": product.state_vectors(),
-            "slant_range_time_to_ground_range": product.slant_range_time_to_ground_range,
-            "correct_radiometry": correct_radiometry,
-        },
-        template=acquisition_template,
-    )
-
-    if correct_radiometry is not None:
-        logger.info("simulate radiometry")
-
-        grid_parameters = product.grid_parameters(grouping_area_factor)
-
-        if correct_radiometry == "gamma_bilinear":
-            gamma_weights = radiometry.gamma_weights_bilinear
-        elif correct_radiometry == "gamma_nearest":
-            gamma_weights = radiometry.gamma_weights_nearest
-
-        acquisition = acquisition.persist()
-
-        simulated_beta_nought = chunking.map_ovelap(
-            obj=acquisition,
-            function=gamma_weights,
-            chunks=radiometry_chunks,
-            bound=radiometry_bound,
-            kwargs=grid_parameters,
-            template=template_raster,
-        )
-        simulated_beta_nought.x.attrs.update(dem_raster.x.attrs)
-        simulated_beta_nought.y.attrs.update(dem_raster.y.attrs)
-        simulated_beta_nought.rio.set_crs(dem_raster.rio.crs)
 
     if simulated_urlpath is not None:
+        assert simulated_beta_nought is not None
         if output_urlpath is not None:
             simulated_beta_nought.persist()
 
@@ -201,36 +263,9 @@ def terrain_correction(
             maybe_delayed.compute()
 
     if output_urlpath is None:
+        assert simulated_beta_nought is not None
         return simulated_beta_nought
 
-    logger.info("calibrate image")
-
-    beta_nought = product.beta_nought()
-
-    logger.info("terrain-correct image")
-
-    if product.product_type == "GRD":
-        interp_kwargs = {"ground_range": acquisition.ground_range}
-    else:
-        interp_kwargs = {"slant_range_time": acquisition.slant_range_time}
-
-    # HACK: we monkey-patch away an optimisation in xr.DataArray.interp that actually makes
-    #   the interpolation much slower when indeces are dask arrays.
-    with mock.patch("xarray.core.missing._localize", lambda o, i: (o, i)):
-        geocoded = beta_nought.interp(
-            method=interp_method,
-            azimuth_time=acquisition.azimuth_time,
-            **interp_kwargs,
-        )
-
-    if correct_radiometry is not None:
-        geocoded = geocoded / simulated_beta_nought
-
-    geocoded.attrs.update(beta_nought.attrs)
-    geocoded.x.attrs.update(dem_raster.x.attrs)
-    geocoded.y.attrs.update(dem_raster.y.attrs)
-    geocoded.rio.set_crs(dem_raster.rio.crs)
-
     logger.info("save output")
 
     maybe_delayed = geocoded.rio.to_raster(
