@@ -1,7 +1,7 @@
 import itertools
+import math
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
-import numpy as np
 import xarray as xr
 
 
@@ -16,10 +16,10 @@ def compute_chunks_1d(
 
     # -bound is needed to avoid to incorporate the last chunk, if smaller of bound in the previous chunk
     if dim_size > bound:
-        number_of_chunks = int(np.ceil((dim_size - bound) / chunks))
+        number_of_chunks = int(math.ceil((dim_size - bound) / chunks))
     else:
         number_of_chunks = 1
-    for n in np.arange(number_of_chunks):
+    for n in range(number_of_chunks):
         l_int = n * chunks
         if n * chunks - bound > 0:
             l_ext = n * chunks - bound
@@ -45,7 +45,6 @@ def compute_chunks_1d(
 def compute_product(
     slices: List[List[slice]], dims_name: List[str]
 ) -> List[Dict[str, slice]]:
-
     product: List[Dict[str, slice]] = []
 
     for slices_ in itertools.product(*slices):
@@ -83,7 +82,6 @@ def map_ovelap(
     kwargs: Dict[Any, Any] = {},
     template: Optional[xr.DataArray] = None,
 ) -> xr.DataArray:
-
     dims = {}
     for d in obj.dims:
         dims[str(d)] = len(obj[d])
@@ -106,7 +104,7 @@ def map_ovelap(
     except ModuleNotFoundError:
         from numpy import empty_like  # type: ignore
 
-    out = xr.DataArray(empty_like(template.data), dims=template.dims)
+    out = xr.DataArray(empty_like(template.data), dims=template.dims)  # type: ignore
     out.coords.update(obj.coords)
     for ext_chunk, ext_chunk_bounds, int_chunk in zip(
         ext_chunks, ext_chunks_bounds, int_chunks
