@@ -9,8 +9,6 @@ import numpy as np
 import numpy.typing as npt
 import xarray as xr
 
-SPEED_OF_LIGHT = 299_792_458.0  # m / s
-
 TimedeltaArrayLike = TypeVar("TimedeltaArrayLike", bound=npt.ArrayLike)
 FloatArrayLike = TypeVar("FloatArrayLike", bound=npt.ArrayLike)
 
@@ -92,12 +90,15 @@ def backward_geocode(
 
     # NOTE: dem_distance has the associated azimuth_time as a coordinate already
     _, _, _, (dem_distance, satellite_direction) = secant_method(
-        zero_doppler, t_prev, t_curr, diff_ufunc  # type: ignore
+        zero_doppler,
+        t_prev,
+        t_curr,
+        diff_ufunc,
     )
     acquisition = xr.Dataset(
         data_vars={
             "dem_distance": dem_distance,
-            "satellite_direction": satellite_direction,
+            "satellite_direction": satellite_direction.transpose(*dem_distance.dims),
         }
     )
     return acquisition.reset_coords("azimuth_time")
