@@ -14,8 +14,10 @@ LOGGER = logging.getLogger(__name__)
 ECEF_CRS = "EPSG:4978"
 
 
-def open_dem_raster(dem_urlpath: str, **kwargs: Any) -> xr.DataArray:
-    dem_raster = xr.open_dataarray(dem_urlpath, engine="rasterio", **kwargs)
+def open_dem_raster(
+    dem_urlpath: str, engine: str = "rasterio", **kwargs: Any
+) -> xr.DataArray:
+    dem_raster = xr.open_dataarray(dem_urlpath, engine=engine, **kwargs)
     if dem_raster.y.diff("y").values[0] < 0:
         dem_raster = dem_raster.isel(y=slice(None, None, -1))
     dem_raster.attrs["long_name"] = "elevation"
@@ -24,24 +26,28 @@ def open_dem_raster(dem_urlpath: str, **kwargs: Any) -> xr.DataArray:
     return dem_raster
 
 
-def convert_to_dem_3d(dem_raster: xr.DataArray, dim: str = "axis") -> xr.DataArray:
-    _, dem_raster_x = xr.broadcast(dem_raster, dem_raster.x)  # type: ignore
-    dem_3d = xr.concat(
-        [dem_raster_x, dem_raster.y, dem_raster], dim=dim, coords="minimal"
-    )
-    if dem_3d.chunks is not None:
-        dem_3d = dem_3d.chunk({dim: None})
+def make_nd_dataarray(das: list[xr.DataArray], dim: str = "axis") -> xr.DataArray:
+    da_nd = xr.concat(das, dim=dim, coords="minimal")
+    return da_nd.assign_coords({dim: range(len(das))})
 
-    dem_3d = dem_3d.assign_coords({dim: [0, 1, 2]})
-    return dem_3d.rename("dem_3d")  # type: ignore
+
+def convert_to_dem_3d(
+    dem_raster: xr.DataArray, dim: str = "axis", x: str = "x", y: str = "y"
+) -> xr.DataArray:
+    _, dem_raster_x = xr.broadcast(dem_raster, dem_raster.coords[x])
+    dem_raster_y = dem_raster.coords[y]
+    dem_3d = make_nd_dataarray([dem_raster_x, dem_raster_y, dem_raster], dim=dim)
+    return dem_3d.rename("dem_3d")
 
 
 def transform_dem_3d(
     dem_3d: xr.DataArray,
-    source_crs: str,
+    source_crs: str | None = None,
     target_crs: str = ECEF_CRS,
     dim: str = "axis",
 ) -> xr.DataArray:
+    if source_crs is None:
+        source_crs = dem_3d.rio.crs
     try:
         x, y, z = warp.transform(
             source_crs,
@@ -68,8 +74,10 @@ def transform_dem_3d(
     return dem_3d_crs
 
 
-def convert_to_dem_ecef(dem_raster: xr.DataArray, **kwargs: Any) -> xr.DataArray:
-    dem_3d = convert_to_dem_3d(dem_raster)
+def convert_to_dem_ecef(
+    dem_raster: xr.DataArray, x: str = "x", y: str = "y", **kwargs: Any
+) -> xr.DataArray:
+    dem_3d = convert_to_dem_3d(dem_raster, x=x, y=y)
     return transform_dem_3d(dem_3d, **kwargs)
 
 
